@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Klaralvdalens Datakonsult AB (KDAB).
+** Copyright (C) 2015 Klaralvdalens Datakonsult AB (KDAB).
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt3D module of the Qt Toolkit.
@@ -37,19 +37,16 @@
 **
 ****************************************************************************/
 
-#include "graphicshelpergl3_2_p.h"
+#include "graphicshelpergl3_3_p.h"
 
 #ifndef QT_OPENGL_ES_2
-#include <QOpenGLFunctions_3_2_Core>
 #include <QOpenGLFunctions_3_3_Core>
 #include <QtOpenGLExtensions/qopenglextensions.h>
 #include <private/attachmentpack_p.h>
 #include <logging_p.h>
 #include <qgraphicsutils_p.h>
 
-QT_BEGIN_NAMESPACE
-
-# ifndef QT_OPENGL_3
+# ifndef QT_OPENGL_3_2
 #  define GL_PATCH_VERTICES 36466
 #  define GL_ACTIVE_RESOURCES 0x92F5
 #  define GL_ACTIVE_UNIFORM_BLOCKS 0x8A36
@@ -68,24 +65,26 @@ QT_BEGIN_NAMESPACE
 #  define GL_UNIFORM_BLOCK_DATA_SIZE 0x8A40
 # endif
 
+QT_BEGIN_NAMESPACE
+
 namespace Qt3DRender {
 namespace Render {
 namespace OpenGL {
 
-GraphicsHelperGL3_2::GraphicsHelperGL3_2()
+GraphicsHelperGL3_3::GraphicsHelperGL3_3()
     : m_funcs(nullptr)
     , m_tessFuncs()
 {
 }
 
-GraphicsHelperGL3_2::~GraphicsHelperGL3_2()
+GraphicsHelperGL3_3::~GraphicsHelperGL3_3()
 {
 }
 
-void GraphicsHelperGL3_2::initializeHelper(QOpenGLContext *context,
-                                           QAbstractOpenGLFunctions *functions)
+void GraphicsHelperGL3_3::initializeHelper(QOpenGLContext *context,
+                                            QAbstractOpenGLFunctions *functions)
 {
-    m_funcs = static_cast<QOpenGLFunctions_3_2_Core*>(functions);
+    m_funcs = static_cast<QOpenGLFunctions_3_3_Core*>(functions);
     const bool ok = m_funcs->initializeOpenGLFunctions();
     Q_ASSERT(ok);
     Q_UNUSED(ok);
@@ -96,7 +95,7 @@ void GraphicsHelperGL3_2::initializeHelper(QOpenGLContext *context,
     }
 }
 
-void GraphicsHelperGL3_2::drawElementsInstancedBaseVertexBaseInstance(GLenum primitiveType,
+void GraphicsHelperGL3_3::drawElementsInstancedBaseVertexBaseInstance(GLenum primitiveType,
                                                                       GLsizei primitiveCount,
                                                                       GLint indexType,
                                                                       void *indices,
@@ -105,7 +104,7 @@ void GraphicsHelperGL3_2::drawElementsInstancedBaseVertexBaseInstance(GLenum pri
                                                                       GLint baseInstance)
 {
     if (baseInstance != 0)
-        qWarning() << "glDrawElementsInstancedBaseVertexBaseInstance is not supported with OpenGL ES 2";
+        qWarning() << "glDrawElementsInstancedBaseVertexBaseInstance is not supported with OpenGL 3.3";
 
     // glDrawElements OpenGL 3.1 or greater
     m_funcs->glDrawElementsInstancedBaseVertex(primitiveType,
@@ -116,10 +115,10 @@ void GraphicsHelperGL3_2::drawElementsInstancedBaseVertexBaseInstance(GLenum pri
                                                baseVertex);
 }
 
-void GraphicsHelperGL3_2::drawArraysInstanced(GLenum primitiveType,
-                                              GLint first,
-                                              GLsizei count,
-                                              GLsizei instances)
+void GraphicsHelperGL3_3::drawArraysInstanced(GLenum primitiveType,
+                                               GLint first,
+                                               GLsizei count,
+                                               GLsizei instances)
 {
     // glDrawArraysInstanced OpenGL 3.1 or greater
     m_funcs->glDrawArraysInstanced(primitiveType,
@@ -128,7 +127,7 @@ void GraphicsHelperGL3_2::drawArraysInstanced(GLenum primitiveType,
                                    instances);
 }
 
-void GraphicsHelperGL3_2::drawArraysInstancedBaseInstance(GLenum primitiveType, GLint first, GLsizei count, GLsizei instances, GLsizei baseInstance)
+void GraphicsHelperGL3_3::drawArraysInstancedBaseInstance(GLenum primitiveType, GLint first, GLsizei count, GLsizei instances, GLsizei baseInstance)
 {
     if (baseInstance != 0)
         qWarning() << "glDrawArraysInstancedBaseInstance is not supported with OpenGL 3";
@@ -138,11 +137,11 @@ void GraphicsHelperGL3_2::drawArraysInstancedBaseInstance(GLenum primitiveType,
                                    instances);
 }
 
-void GraphicsHelperGL3_2::drawElements(GLenum primitiveType,
-                                       GLsizei primitiveCount,
-                                       GLint indexType,
-                                       void *indices,
-                                       GLint baseVertex)
+void GraphicsHelperGL3_3::drawElements(GLenum primitiveType,
+                                        GLsizei primitiveCount,
+                                        GLint indexType,
+                                        void *indices,
+                                        GLint baseVertex)
 {
     m_funcs->glDrawElementsBaseVertex(primitiveType,
                                       primitiveCount,
@@ -151,26 +150,26 @@ void GraphicsHelperGL3_2::drawElements(GLenum primitiveType,
                                       baseVertex);
 }
 
-void GraphicsHelperGL3_2::drawArrays(GLenum primitiveType,
-                                     GLint first,
-                                     GLsizei count)
+void GraphicsHelperGL3_3::drawElementsIndirect(GLenum, GLenum, void *)
+{
+    qWarning() << "Indirect Drawing is not supported with OpenGL 3";
+}
+
+void GraphicsHelperGL3_3::drawArrays(GLenum primitiveType,
+                                      GLint first,
+                                      GLsizei count)
 {
     m_funcs->glDrawArrays(primitiveType,
                           first,
                           count);
 }
 
-void GraphicsHelperGL3_2::drawElementsIndirect(GLenum, GLenum, void *)
+void GraphicsHelperGL3_3::drawArraysIndirect(GLenum , void *)
 {
-    qWarning() << "Indirect Drawing is not supported with OpenGL 3.2";
+    qWarning() << "Indirect Drawing is not supported with OpenGL 3";
 }
 
-void GraphicsHelperGL3_2::drawArraysIndirect(GLenum , void *)
-{
-    qWarning() << "Indirect Drawing is not supported with OpenGL 3.2";
-}
-
-void GraphicsHelperGL3_2::setVerticesPerPatch(GLint verticesPerPatch)
+void GraphicsHelperGL3_3::setVerticesPerPatch(GLint verticesPerPatch)
 {
 #if defined(QT_OPENGL_4)
     if (!m_tessFuncs) {
@@ -185,12 +184,12 @@ void GraphicsHelperGL3_2::setVerticesPerPatch(GLint verticesPerPatch)
 #endif
 }
 
-void GraphicsHelperGL3_2::useProgram(GLuint programId)
+void GraphicsHelperGL3_3::useProgram(GLuint programId)
 {
     m_funcs->glUseProgram(programId);
 }
 
-QVector<ShaderUniform> GraphicsHelperGL3_2::programUniformsAndLocations(GLuint programId)
+QVector<ShaderUniform> GraphicsHelperGL3_3::programUniformsAndLocations(GLuint programId)
 {
     QVector<ShaderUniform> uniforms;
 
@@ -225,7 +224,7 @@ QVector<ShaderUniform> GraphicsHelperGL3_2::programUniformsAndLocations(GLuint p
     return uniforms;
 }
 
-QVector<ShaderAttribute> GraphicsHelperGL3_2::programAttributesAndLocations(GLuint programId)
+QVector<ShaderAttribute> GraphicsHelperGL3_3::programAttributesAndLocations(GLuint programId)
 {
     QVector<ShaderAttribute> attributes;
     GLint nbrActiveAttributes = 0;
@@ -247,7 +246,7 @@ QVector<ShaderAttribute> GraphicsHelperGL3_2::programAttributesAndLocations(GLui
     return attributes;
 }
 
-QVector<ShaderUniformBlock> GraphicsHelperGL3_2::programUniformBlocks(GLuint programId)
+QVector<ShaderUniformBlock> GraphicsHelperGL3_3::programUniformBlocks(GLuint programId)
 {
     QVector<ShaderUniformBlock> blocks;
     GLint nbrActiveUniformsBlocks = 0;
@@ -268,22 +267,20 @@ QVector<ShaderUniformBlock> GraphicsHelperGL3_2::programUniformBlocks(GLuint pro
     return blocks;
 }
 
-QVector<ShaderStorageBlock> GraphicsHelperGL3_2::programShaderStorageBlocks(GLuint programId)
+QVector<ShaderStorageBlock> GraphicsHelperGL3_3::programShaderStorageBlocks(GLuint programId)
 {
     Q_UNUSED(programId);
     QVector<ShaderStorageBlock> blocks;
-    qWarning() << "SSBO are not supported by OpenGL 3.2 (since OpenGL 4.3)";
+    qWarning() << "SSBO are not supported by OpenGL 3.3 (since OpenGL 4.3)";
     return blocks;
 }
 
-void GraphicsHelperGL3_2::vertexAttribDivisor(GLuint index, GLuint divisor)
+void GraphicsHelperGL3_3::vertexAttribDivisor(GLuint index, GLuint divisor)
 {
-    Q_UNUSED(index);
-    Q_UNUSED(divisor);
-    qCWarning(Rendering) << "Vertex attribute divisor not available with OpenGL 3.2 core";
+    m_funcs->glVertexAttribDivisor(index, divisor);
 }
 
-void GraphicsHelperGL3_2::vertexAttributePointer(GLenum shaderDataType,
+void GraphicsHelperGL3_3::vertexAttributePointer(GLenum shaderDataType,
                                                  GLuint index,
                                                  GLint size,
                                                  GLenum type,
@@ -321,36 +318,35 @@ void GraphicsHelperGL3_2::vertexAttributePointer(GLenum shaderDataType,
 
     default:
         qCWarning(Rendering) << "vertexAttribPointer: Unhandled type";
-        Q_UNREACHABLE();
     }
 }
 
-void GraphicsHelperGL3_2::readBuffer(GLenum mode)
+void GraphicsHelperGL3_3::readBuffer(GLenum mode)
 {
     m_funcs->glReadBuffer(mode);
 }
 
-void GraphicsHelperGL3_2::drawBuffer(GLenum mode)
+void GraphicsHelperGL3_3::drawBuffer(GLenum mode)
 {
     m_funcs->glDrawBuffer(mode);
 }
 
-void *GraphicsHelperGL3_2::fenceSync()
+void *GraphicsHelperGL3_3::fenceSync()
 {
     return m_funcs->glFenceSync(GL_SYNC_GPU_COMMANDS_COMPLETE, 0);
 }
 
-void GraphicsHelperGL3_2::clientWaitSync(void *sync, GLuint64 nanoSecTimeout)
+void GraphicsHelperGL3_3::clientWaitSync(void *sync, GLuint64 nanoSecTimeout)
 {
     m_funcs->glClientWaitSync(static_cast<GLsync>(sync), GL_SYNC_FLUSH_COMMANDS_BIT, nanoSecTimeout);
 }
 
-void GraphicsHelperGL3_2::waitSync(void *sync)
+void GraphicsHelperGL3_3::waitSync(void *sync)
 {
     m_funcs->glWaitSync(static_cast<GLsync>(sync), 0, GL_TIMEOUT_IGNORED);
 }
 
-bool GraphicsHelperGL3_2::wasSyncSignaled(void *sync)
+bool GraphicsHelperGL3_3::wasSyncSignaled(void *sync)
 {
     GLint v;
     m_funcs->glGetSynciv(static_cast<GLsync>(sync),
@@ -361,31 +357,31 @@ bool GraphicsHelperGL3_2::wasSyncSignaled(void *sync)
     return v == GL_SIGNALED;
 }
 
-void GraphicsHelperGL3_2::deleteSync(void *sync)
+void GraphicsHelperGL3_3::deleteSync(void *sync)
 {
     m_funcs->glDeleteSync(static_cast<GLsync>(sync));
 }
 
-void GraphicsHelperGL3_2::rasterMode(GLenum faceMode, GLenum rasterMode)
+void GraphicsHelperGL3_3::rasterMode(GLenum faceMode, GLenum rasterMode)
 {
     m_funcs->glPolygonMode(faceMode, rasterMode);
 }
 
-void GraphicsHelperGL3_2::blendEquation(GLenum mode)
+void GraphicsHelperGL3_3::blendEquation(GLenum mode)
 {
     m_funcs->glBlendEquation(mode);
 }
 
-void GraphicsHelperGL3_2::blendFunci(GLuint buf, GLenum sfactor, GLenum dfactor)
+void GraphicsHelperGL3_3::blendFunci(GLuint buf, GLenum sfactor, GLenum dfactor)
 {
     Q_UNUSED(buf);
     Q_UNUSED(sfactor);
     Q_UNUSED(dfactor);
 
-    qWarning() << "glBlendFunci() not supported by OpenGL 3.0 (since OpenGL 4.0)";
+    qWarning() << "glBlendFunci() not supported by OpenGL 3.3 (since OpenGL 4.0)";
 }
 
-void GraphicsHelperGL3_2::blendFuncSeparatei(GLuint buf, GLenum sRGB, GLenum dRGB, GLenum sAlpha, GLenum dAlpha)
+void GraphicsHelperGL3_3::blendFuncSeparatei(GLuint buf, GLenum sRGB, GLenum dRGB, GLenum sAlpha, GLenum dAlpha)
 {
     Q_UNUSED(buf);
     Q_UNUSED(sRGB);
@@ -393,61 +389,61 @@ void GraphicsHelperGL3_2::blendFuncSeparatei(GLuint buf, GLenum sRGB, GLenum dRG
     Q_UNUSED(sAlpha);
     Q_UNUSED(dAlpha);
 
-    qWarning() << "glBlendFuncSeparatei() not supported by OpenGL 3.0 (since OpenGL 4.0)";
+    qWarning() << "glBlendFuncSeparatei() not supported by OpenGL 3.3 (since OpenGL 4.0)";
 }
 
-void GraphicsHelperGL3_2::alphaTest(GLenum, GLenum)
+void GraphicsHelperGL3_3::alphaTest(GLenum, GLenum)
 {
     qCWarning(Rendering) << "AlphaTest not available with OpenGL 3.2 core";
 }
 
-void GraphicsHelperGL3_2::depthTest(GLenum mode)
+void GraphicsHelperGL3_3::depthTest(GLenum mode)
 {
     m_funcs->glEnable(GL_DEPTH_TEST);
     m_funcs->glDepthFunc(mode);
 }
 
-void GraphicsHelperGL3_2::depthMask(GLenum mode)
+void GraphicsHelperGL3_3::depthMask(GLenum mode)
 {
     m_funcs->glDepthMask(mode);
 }
 
-void GraphicsHelperGL3_2::depthRange(GLdouble nearValue, GLdouble farValue)
+void GraphicsHelperGL3_3::depthRange(GLdouble nearValue, GLdouble farValue)
 {
     m_funcs->glDepthRange(nearValue, farValue);
 }
 
-void GraphicsHelperGL3_2::frontFace(GLenum mode)
+void GraphicsHelperGL3_3::frontFace(GLenum mode)
 {
     m_funcs->glFrontFace(mode);
 
 }
 
-void GraphicsHelperGL3_2::setMSAAEnabled(bool enabled)
+void GraphicsHelperGL3_3::setMSAAEnabled(bool enabled)
 {
     enabled ? m_funcs->glEnable(GL_MULTISAMPLE)
             : m_funcs->glDisable(GL_MULTISAMPLE);
 }
 
-void GraphicsHelperGL3_2::setAlphaCoverageEnabled(bool enabled)
+void GraphicsHelperGL3_3::setAlphaCoverageEnabled(bool enabled)
 {
     enabled ? m_funcs->glEnable(GL_SAMPLE_ALPHA_TO_COVERAGE)
             : m_funcs->glDisable(GL_SAMPLE_ALPHA_TO_COVERAGE);
 }
 
-GLuint GraphicsHelperGL3_2::createFrameBufferObject()
+GLuint GraphicsHelperGL3_3::createFrameBufferObject()
 {
     GLuint id;
     m_funcs->glGenFramebuffers(1, &id);
     return id;
 }
 
-void GraphicsHelperGL3_2::releaseFrameBufferObject(GLuint frameBufferId)
+void GraphicsHelperGL3_3::releaseFrameBufferObject(GLuint frameBufferId)
 {
     m_funcs->glDeleteFramebuffers(1, &frameBufferId);
 }
 
-void GraphicsHelperGL3_2::bindFrameBufferObject(GLuint frameBufferId, FBOBindMode mode)
+void GraphicsHelperGL3_3::bindFrameBufferObject(GLuint frameBufferId, FBOBindMode mode)
 {
     switch (mode) {
     case FBODraw:
@@ -463,40 +459,25 @@ void GraphicsHelperGL3_2::bindFrameBufferObject(GLuint frameBufferId, FBOBindMod
     }
 }
 
-void GraphicsHelperGL3_2::bindImageTexture(GLuint imageUnit, GLuint texture,
-                                           GLint mipLevel, GLboolean layered,
-                                           GLint layer, GLenum access, GLenum format)
-{
-    Q_UNUSED(imageUnit)
-    Q_UNUSED(texture)
-    Q_UNUSED(mipLevel)
-    Q_UNUSED(layered)
-    Q_UNUSED(layer)
-    Q_UNUSED(access)
-    Q_UNUSED(format)
-    qWarning() << "Shader Images are not supported by OpenGL 3.2 (since OpenGL 4.2)";
-
-}
-
-GLuint GraphicsHelperGL3_2::boundFrameBufferObject()
+GLuint GraphicsHelperGL3_3::boundFrameBufferObject()
 {
     GLint id = 0;
     m_funcs->glGetIntegerv(GL_DRAW_FRAMEBUFFER_BINDING, &id);
     return id;
 }
 
-bool GraphicsHelperGL3_2::checkFrameBufferComplete()
+bool GraphicsHelperGL3_3::checkFrameBufferComplete()
 {
     return (m_funcs->glCheckFramebufferStatus(GL_FRAMEBUFFER) == GL_FRAMEBUFFER_COMPLETE);
 }
 
-bool GraphicsHelperGL3_2::frameBufferNeedsRenderBuffer(const Attachment &attachment)
+bool GraphicsHelperGL3_3::frameBufferNeedsRenderBuffer(const Attachment &attachment)
 {
     Q_UNUSED(attachment);
     return false;
 }
 
-void GraphicsHelperGL3_2::bindFrameBufferAttachment(QOpenGLTexture *texture, const Attachment &attachment)
+void GraphicsHelperGL3_3::bindFrameBufferAttachment(QOpenGLTexture *texture, const Attachment &attachment)
 {
     GLenum attr = GL_DEPTH_STENCIL_ATTACHMENT;
 
@@ -512,23 +493,23 @@ void GraphicsHelperGL3_2::bindFrameBufferAttachment(QOpenGLTexture *texture, con
     if (target == QOpenGLTexture::Target1DArray || target == QOpenGLTexture::Target2DArray ||
             target == QOpenGLTexture::Target2DMultisampleArray || target == QOpenGLTexture::Target3D)
         m_funcs->glFramebufferTextureLayer(GL_DRAW_FRAMEBUFFER, attr, texture->textureId(), attachment.m_mipLevel, attachment.m_layer);
-    else if (target == QOpenGLTexture::TargetCubeMapArray)
-        m_funcs->glFramebufferTexture3D(GL_DRAW_FRAMEBUFFER, attr, attachment.m_face, texture->textureId(), attachment.m_mipLevel, attachment.m_layer);
-    else if (target == QOpenGLTexture::TargetCubeMap)
+    else if (target == QOpenGLTexture::TargetCubeMapArray && attachment.m_face != QAbstractTexture::AllFaces)
+        m_funcs->glFramebufferTextureLayer( GL_DRAW_FRAMEBUFFER, attr, texture->textureId(), attachment.m_mipLevel, attachment.m_layer * 6 + (attachment.m_face - QAbstractTexture::CubeMapPositiveX));
+    else if (target == QOpenGLTexture::TargetCubeMap && attachment.m_face != QAbstractTexture::AllFaces)
         m_funcs->glFramebufferTexture2D(GL_DRAW_FRAMEBUFFER, attr, attachment.m_face, texture->textureId(), attachment.m_mipLevel);
     else
         m_funcs->glFramebufferTexture(GL_DRAW_FRAMEBUFFER, attr, texture->textureId(), attachment.m_mipLevel);
     texture->release();
 }
 
-void GraphicsHelperGL3_2::bindFrameBufferAttachment(RenderBuffer *renderBuffer, const Attachment &attachment)
+void GraphicsHelperGL3_3::bindFrameBufferAttachment(RenderBuffer *renderBuffer, const Attachment &attachment)
 {
     Q_UNUSED(renderBuffer);
     Q_UNUSED(attachment);
     Q_UNREACHABLE();
 }
 
-bool GraphicsHelperGL3_2::supportsFeature(GraphicsHelperInterface::Feature feature) const
+bool GraphicsHelperGL3_3::supportsFeature(GraphicsHelperInterface::Feature feature) const
 {
     switch (feature) {
     case MRT:
@@ -547,7 +528,7 @@ bool GraphicsHelperGL3_2::supportsFeature(GraphicsHelperInterface::Feature featu
     }
 }
 
-void GraphicsHelperGL3_2::drawBuffers(GLsizei n, const int *bufs)
+void GraphicsHelperGL3_3::drawBuffers(GLsizei n, const int *bufs)
 {
     // Use QVarLengthArray here
     QVarLengthArray<GLenum, 16> drawBufs(n);
@@ -557,31 +538,45 @@ void GraphicsHelperGL3_2::drawBuffers(GLsizei n, const int *bufs)
     m_funcs->glDrawBuffers(n, drawBufs.constData());
 }
 
-void GraphicsHelperGL3_2::bindFragDataLocation(GLuint shader, const QHash<QString, int> &outputs)
+void GraphicsHelperGL3_3::bindFragDataLocation(GLuint shader, const QHash<QString, int> &outputs)
 {
     for (auto it = outputs.begin(), end = outputs.end(); it != end; ++it)
         m_funcs->glBindFragDataLocation(shader, it.value(), it.key().toStdString().c_str());
 }
 
-void GraphicsHelperGL3_2::bindUniformBlock(GLuint programId, GLuint uniformBlockIndex, GLuint uniformBlockBinding)
+void GraphicsHelperGL3_3::bindUniformBlock(GLuint programId, GLuint uniformBlockIndex, GLuint uniformBlockBinding)
 {
     m_funcs->glUniformBlockBinding(programId, uniformBlockIndex, uniformBlockBinding);
 }
 
-void GraphicsHelperGL3_2::bindShaderStorageBlock(GLuint programId, GLuint shaderStorageBlockIndex, GLuint shaderStorageBlockBinding)
+void GraphicsHelperGL3_3::bindShaderStorageBlock(GLuint programId, GLuint shaderStorageBlockIndex, GLuint shaderStorageBlockBinding)
 {
     Q_UNUSED(programId);
     Q_UNUSED(shaderStorageBlockIndex);
     Q_UNUSED(shaderStorageBlockBinding);
-    qWarning() << "SSBO are not supported by OpenGL 3.0 (since OpenGL 4.3)";
+    qWarning() << "SSBO are not supported by OpenGL 3.3 (since OpenGL 4.3)";
 }
 
-void GraphicsHelperGL3_2::bindBufferBase(GLenum target, GLuint index, GLuint buffer)
+void GraphicsHelperGL3_3::bindImageTexture(GLuint imageUnit, GLuint texture,
+                                           GLint mipLevel, GLboolean layered,
+                                           GLint layer, GLenum access, GLenum format)
+{
+    Q_UNUSED(imageUnit)
+    Q_UNUSED(texture)
+    Q_UNUSED(mipLevel)
+    Q_UNUSED(layered)
+    Q_UNUSED(layer)
+    Q_UNUSED(access)
+    Q_UNUSED(format)
+    qWarning() << "Shader Images are not supported by OpenGL 3.3 (since OpenGL 4.2)";
+}
+
+void GraphicsHelperGL3_3::bindBufferBase(GLenum target, GLuint index, GLuint buffer)
 {
     m_funcs->glBindBufferBase(target, index, buffer);
 }
 
-void GraphicsHelperGL3_2::buildUniformBuffer(const QVariant &v, const ShaderUniform &description, QByteArray &buffer)
+void GraphicsHelperGL3_3::buildUniformBuffer(const QVariant &v, const ShaderUniform &description, QByteArray &buffer)
 {
     char *bufferData = buffer.data();
 
@@ -785,7 +780,7 @@ void GraphicsHelperGL3_2::buildUniformBuffer(const QVariant &v, const ShaderUnif
     }
 }
 
-uint GraphicsHelperGL3_2::uniformByteSize(const ShaderUniform &description)
+uint GraphicsHelperGL3_3::uniformByteSize(const ShaderUniform &description)
 {
     uint rawByteSize = 0;
     int arrayStride = qMax(description.m_arrayStride, 0);
@@ -909,17 +904,17 @@ uint GraphicsHelperGL3_2::uniformByteSize(const ShaderUniform &description)
     return arrayStride ? rawByteSize * arrayStride : rawByteSize;
 }
 
-void GraphicsHelperGL3_2::enableClipPlane(int clipPlane)
+void GraphicsHelperGL3_3::enableClipPlane(int clipPlane)
 {
     m_funcs->glEnable(GL_CLIP_DISTANCE0 + clipPlane);
 }
 
-void GraphicsHelperGL3_2::disableClipPlane(int clipPlane)
+void GraphicsHelperGL3_3::disableClipPlane(int clipPlane)
 {
     m_funcs->glDisable(GL_CLIP_DISTANCE0 + clipPlane);
 }
 
-void GraphicsHelperGL3_2::setClipPlane(int clipPlane, const QVector3D &normal, float distance)
+void GraphicsHelperGL3_3::setClipPlane(int clipPlane, const QVector3D &normal, float distance)
 {
     // deprecated
     Q_UNUSED(clipPlane);
@@ -927,42 +922,42 @@ void GraphicsHelperGL3_2::setClipPlane(int clipPlane, const QVector3D &normal, f
     Q_UNUSED(distance);
 }
 
-GLint GraphicsHelperGL3_2::maxClipPlaneCount()
+GLint GraphicsHelperGL3_3::maxClipPlaneCount()
 {
     GLint max = 0;
     m_funcs->glGetIntegerv(GL_MAX_CLIP_DISTANCES, &max);
     return max;
 }
 
-void GraphicsHelperGL3_2::memoryBarrier(QMemoryBarrier::Operations barriers)
+void GraphicsHelperGL3_3::memoryBarrier(QMemoryBarrier::Operations barriers)
 {
     Q_UNUSED(barriers);
-    qWarning() << "memory barrier is not supported by OpenGL 3.0 (since 4.3)";
+    qWarning() << "memory barrier is not supported by OpenGL 3.3 (since 4.3)";
 }
 
-void GraphicsHelperGL3_2::enablePrimitiveRestart(int primitiveRestartIndex)
+void GraphicsHelperGL3_3::enablePrimitiveRestart(int primitiveRestartIndex)
 {
     m_funcs->glPrimitiveRestartIndex(primitiveRestartIndex);
     m_funcs->glEnable(GL_PRIMITIVE_RESTART);
 }
 
-void GraphicsHelperGL3_2::enableVertexAttributeArray(int location)
+void GraphicsHelperGL3_3::enableVertexAttributeArray(int location)
 {
     m_funcs->glEnableVertexAttribArray(location);
 }
 
-void GraphicsHelperGL3_2::disablePrimitiveRestart()
+void GraphicsHelperGL3_3::disablePrimitiveRestart()
 {
     m_funcs->glDisable(GL_PRIMITIVE_RESTART);
 }
 
-void GraphicsHelperGL3_2::clearBufferf(GLint drawbuffer, const QVector4D &values)
+void GraphicsHelperGL3_3::clearBufferf(GLint drawbuffer, const QVector4D &values)
 {
     GLfloat vec[4] = {values[0], values[1], values[2], values[3]};
     m_funcs->glClearBufferfv(GL_COLOR, drawbuffer, vec);
 }
 
-void GraphicsHelperGL3_2::pointSize(bool programmable, GLfloat value)
+void GraphicsHelperGL3_3::pointSize(bool programmable, GLfloat value)
 {
     if (programmable) {
         m_funcs->glEnable(GL_PROGRAM_POINT_SIZE);
@@ -972,17 +967,17 @@ void GraphicsHelperGL3_2::pointSize(bool programmable, GLfloat value)
     }
 }
 
-void GraphicsHelperGL3_2::enablei(GLenum cap, GLuint index)
+void GraphicsHelperGL3_3::enablei(GLenum cap, GLuint index)
 {
     m_funcs->glEnablei(cap, index);
 }
 
-void GraphicsHelperGL3_2::disablei(GLenum cap, GLuint index)
+void GraphicsHelperGL3_3::disablei(GLenum cap, GLuint index)
 {
     m_funcs->glDisablei(cap, index);
 }
 
-void GraphicsHelperGL3_2::setSeamlessCubemap(bool enable)
+void GraphicsHelperGL3_3::setSeamlessCubemap(bool enable)
 {
     if (enable)
         m_funcs->glEnable(GL_TEXTURE_CUBE_MAP_SEAMLESS);
@@ -990,7 +985,7 @@ void GraphicsHelperGL3_2::setSeamlessCubemap(bool enable)
         m_funcs->glDisable(GL_TEXTURE_CUBE_MAP_SEAMLESS);
 }
 
-QSize GraphicsHelperGL3_2::getRenderBufferDimensions(GLuint renderBufferId)
+QSize GraphicsHelperGL3_3::getRenderBufferDimensions(GLuint renderBufferId)
 {
     GLint width = 0;
     GLint height = 0;
@@ -1003,7 +998,7 @@ QSize GraphicsHelperGL3_2::getRenderBufferDimensions(GLuint renderBufferId)
     return QSize(width, height);
 }
 
-QSize GraphicsHelperGL3_2::getTextureDimensions(GLuint textureId, GLenum target, uint level)
+QSize GraphicsHelperGL3_3::getTextureDimensions(GLuint textureId, GLenum target, uint level)
 {
     GLint width = 0;
     GLint height = 0;
@@ -1016,130 +1011,130 @@ QSize GraphicsHelperGL3_2::getTextureDimensions(GLuint textureId, GLenum target,
     return QSize(width, height);
 }
 
-void GraphicsHelperGL3_2::dispatchCompute(GLuint wx, GLuint wy, GLuint wz)
+void GraphicsHelperGL3_3::dispatchCompute(GLuint wx, GLuint wy, GLuint wz)
 {
     Q_UNUSED(wx);
     Q_UNUSED(wy);
     Q_UNUSED(wz);
-    qWarning() << "Compute Shaders are not supported by OpenGL 3.2 (since OpenGL 4.3)";
+    qWarning() << "Compute Shaders are not supported by OpenGL 3.3 (since OpenGL 4.3)";
 }
 
-char *GraphicsHelperGL3_2::mapBuffer(GLenum target, GLsizeiptr size)
+char *GraphicsHelperGL3_3::mapBuffer(GLenum target, GLsizeiptr size)
 {
     return static_cast<char*>(m_funcs->glMapBufferRange(target, 0, size, GL_MAP_READ_BIT | GL_MAP_WRITE_BIT));
 }
 
-GLboolean GraphicsHelperGL3_2::unmapBuffer(GLenum target)
+GLboolean GraphicsHelperGL3_3::unmapBuffer(GLenum target)
 {
     return m_funcs->glUnmapBuffer(target);
 }
 
-void GraphicsHelperGL3_2::glUniform1fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniform1fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniform1fv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform2fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniform2fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniform2fv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform3fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniform3fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniform3fv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform4fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniform4fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniform4fv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform1iv(GLint location, GLsizei count, const GLint *values)
+void GraphicsHelperGL3_3::glUniform1iv(GLint location, GLsizei count, const GLint *values)
 {
     m_funcs->glUniform1iv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform2iv(GLint location, GLsizei count, const GLint *values)
+void GraphicsHelperGL3_3::glUniform2iv(GLint location, GLsizei count, const GLint *values)
 {
     m_funcs->glUniform2iv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform3iv(GLint location, GLsizei count, const GLint *values)
+void GraphicsHelperGL3_3::glUniform3iv(GLint location, GLsizei count, const GLint *values)
 {
     m_funcs->glUniform3iv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform4iv(GLint location, GLsizei count, const GLint *values)
+void GraphicsHelperGL3_3::glUniform4iv(GLint location, GLsizei count, const GLint *values)
 {
     m_funcs->glUniform4iv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform1uiv(GLint location, GLsizei count, const GLuint *values)
+void GraphicsHelperGL3_3::glUniform1uiv(GLint location, GLsizei count, const GLuint *values)
 {
     m_funcs->glUniform1uiv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform2uiv(GLint location, GLsizei count, const GLuint *values)
+void GraphicsHelperGL3_3::glUniform2uiv(GLint location, GLsizei count, const GLuint *values)
 {
     m_funcs->glUniform2uiv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform3uiv(GLint location, GLsizei count, const GLuint *values)
+void GraphicsHelperGL3_3::glUniform3uiv(GLint location, GLsizei count, const GLuint *values)
 {
     m_funcs->glUniform3uiv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniform4uiv(GLint location, GLsizei count, const GLuint *values)
+void GraphicsHelperGL3_3::glUniform4uiv(GLint location, GLsizei count, const GLuint *values)
 {
     m_funcs->glUniform4uiv(location, count, values);
 }
 
-void GraphicsHelperGL3_2::glUniformMatrix2fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniformMatrix2fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniformMatrix2fv(location, count, false, values);
 }
 
-void GraphicsHelperGL3_2::glUniformMatrix3fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniformMatrix3fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniformMatrix3fv(location, count, false, values);
 }
 
-void GraphicsHelperGL3_2::glUniformMatrix4fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniformMatrix4fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniformMatrix4fv(location, count, false, values);
 }
 
-void GraphicsHelperGL3_2::glUniformMatrix2x3fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniformMatrix2x3fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniformMatrix2x3fv(location, count, false, values);
 }
 
-void GraphicsHelperGL3_2::glUniformMatrix3x2fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniformMatrix3x2fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniformMatrix3x2fv(location, count, false, values);
 }
 
-void GraphicsHelperGL3_2::glUniformMatrix2x4fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniformMatrix2x4fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniformMatrix2x4fv(location, count, false, values);
 }
 
-void GraphicsHelperGL3_2::glUniformMatrix4x2fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniformMatrix4x2fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniformMatrix4x2fv(location, count, false, values);
 }
 
-void GraphicsHelperGL3_2::glUniformMatrix3x4fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniformMatrix3x4fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniformMatrix3x4fv(location, count, false, values);
 }
 
-void GraphicsHelperGL3_2::glUniformMatrix4x3fv(GLint location, GLsizei count, const GLfloat *values)
+void GraphicsHelperGL3_3::glUniformMatrix4x3fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniformMatrix4x3fv(location, count, false, values);
 }
 
-UniformType GraphicsHelperGL3_2::uniformTypeFromGLType(GLenum type)
+UniformType GraphicsHelperGL3_3::uniformTypeFromGLType(GLenum type)
 {
     switch (type) {
     case GL_FLOAT:
@@ -1196,13 +1191,13 @@ UniformType GraphicsHelperGL3_2::uniformTypeFromGLType(GLenum type)
     case GL_SAMPLER_BUFFER:
     case GL_SAMPLER_1D:
     case GL_SAMPLER_1D_SHADOW:
+    case GL_SAMPLER_1D_ARRAY:
     case GL_SAMPLER_2D:
     case GL_SAMPLER_2D_RECT:
     case GL_SAMPLER_2D_SHADOW:
     case GL_SAMPLER_2D_RECT_SHADOW:
     case GL_SAMPLER_CUBE:
     case GL_SAMPLER_CUBE_SHADOW:
-    case GL_SAMPLER_1D_ARRAY:
     case GL_SAMPLER_2D_ARRAY:
     case GL_SAMPLER_2D_ARRAY_SHADOW:
     case GL_SAMPLER_2D_MULTISAMPLE:
@@ -1233,7 +1228,7 @@ UniformType GraphicsHelperGL3_2::uniformTypeFromGLType(GLenum type)
     }
 }
 
-void GraphicsHelperGL3_2::blitFramebuffer(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
+void GraphicsHelperGL3_3::blitFramebuffer(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
 {
     m_funcs->glBlitFramebuffer(srcX0, srcY0, srcX1, srcY1, dstX0, dstY0, dstX1, dstY1, mask, filter);
 }
