@@ -26,16 +26,19 @@
 **
 ****************************************************************************/
 
+// TODO Remove in Qt6
+#include <QtCore/qcompilerdetection.h>
+QT_WARNING_DISABLE_DEPRECATED
+
 #include <QtTest/QtTest>
 #include <Qt3DRender/private/entity_p.h>
+#include <Qt3DRender/private/entity_p_p.h>
 #include <Qt3DRender/private/nodemanagers_p.h>
 #include <Qt3DRender/private/managers_p.h>
+#include <Qt3DRender/private/entityvisitor_p.h>
+#include <Qt3DRender/private/entityaccumulator_p.h>
 
 #include <Qt3DRender/QCameraLens>
-#include <Qt3DCore/QPropertyUpdatedChange>
-#include <Qt3DCore/QPropertyNodeAddedChange>
-#include <Qt3DCore/QComponentAddedChange>
-#include <Qt3DCore/QComponentRemovedChange>
 #include <Qt3DCore/QTransform>
 
 #include <Qt3DRender/QEnvironmentLight>
@@ -69,6 +72,24 @@ QVector<QNodeId> layersUuid(Entity *entity) { return entity->componentsUuid<Laye
 QVector<QNodeId> shadersUuid(Entity *entity) { return entity->componentsUuid<ShaderData>(); }
 QVector<QNodeId> environmentLightsUuid(Entity *entity) { return entity->componentsUuid<EnvironmentLight>(); }
 
+class CompleteVisitor : public EntityVisitor
+{
+public:
+    CompleteVisitor(NodeManagers *manager) : EntityVisitor(manager) { }
+
+    int count = 0;
+    Operation visit(Entity *) { count++; return Continue; }
+};
+
+class EnabledVisitor : public EntityVisitor
+{
+public:
+    EnabledVisitor(NodeManagers *manager) : EntityVisitor(manager) { }
+
+    int count = 0;
+    Operation visit(Entity *e) { count++; return e->isEnabled() ? Continue : Prune; }
+};
+
 class tst_RenderEntity : public QObject
 {
     Q_OBJECT
@@ -100,15 +121,17 @@ private slots:
     void checkInitialAndCleanUpState()
     {
         // GIVEN
-        QFETCH(QList<QComponent*>, components);
+        QFETCH(const QList<QComponent*>, components);
 
         TestRenderer renderer;
         NodeManagers nodeManagers;
         Qt3DRender::Render::Entity entity;
-        Qt3DCore::QEntity dummyFrontendEntity;
         entity.setRenderer(&renderer);
         entity.setNodeManagers(&nodeManagers);
 
+        // THEN
+        QCOMPARE(renderer.dirtyBits(), 0);
+
         // THEN
         QVERIFY(entity.componentUuid<Transform>().isNull());
         QVERIFY(entity.componentUuid<CameraLens>().isNull());
@@ -123,21 +146,16 @@ private slots:
         QVERIFY(!entity.isBoundingVolumeDirty());
         QVERIFY(entity.childrenHandles().isEmpty());
         QVERIFY(entity.layerIds().isEmpty());
+        QCOMPARE(entity.renderer(), &renderer);
+
+        QCOMPARE(renderer.dirtyBits(), 0);
 
         // WHEN
-        Q_FOREACH (QComponent *component, components) {
-            const auto addChange = QComponentAddedChangePtr::create(&dummyFrontendEntity, component);
-            entity.sceneChangeEvent(addChange);
-        }
-
-        Qt3DCore::QEntity dummyFrontendEntityChild;
-        // Create nodes in the backend manager
-        nodeManagers.renderNodesManager()->getOrCreateResource(dummyFrontendEntity.id());
-        nodeManagers.renderNodesManager()->getOrCreateResource(dummyFrontendEntityChild.id());
-        // Send children added event to entity
-        const auto addEntityChange = QPropertyNodeAddedChangePtr::create(dummyFrontendEntity.id(), &dummyFrontendEntityChild);
-        entity.sceneChangeEvent(addEntityChange);
+        for (QComponent *component : components)
+            EntityPrivate::get(&entity)->componentAdded(component);
 
+        QVERIFY(renderer.dirtyBits() & Qt3DRender::Render::AbstractRenderer::AllDirty);
+        renderer.resetDirty();
 
         // THEN
         QVERIFY(!entity.componentUuid<Transform>().isNull());
@@ -153,7 +171,7 @@ private slots:
         QVERIFY(entity.isBoundingVolumeDirty());
         QVERIFY(entity.childrenHandles().isEmpty());
         QVERIFY(!entity.layerIds().isEmpty());
-        QVERIFY(renderer.dirtyBits() != 0);
+        QCOMPARE(renderer.dirtyBits(), 0);
         bool containsAll = entity.containsComponentsOfType<Transform,
                 CameraLens, Material, GeometryRenderer, ObjectPicker, ComputeCommand, Armature>();
         QVERIFY(containsAll);
@@ -162,7 +180,6 @@ private slots:
         entity.cleanup();
 
         // THEN
-        QVERIFY(entity.parentEntityId().isNull());
         QVERIFY(entity.componentUuid<Transform>().isNull());
         QVERIFY(entity.componentUuid<CameraLens>().isNull());
         QVERIFY(entity.componentUuid<Material>().isNull());
@@ -179,31 +196,23 @@ private slots:
         containsAll = entity.containsComponentsOfType<Transform,
                 CameraLens, Material, GeometryRenderer, ObjectPicker, ComputeCommand, Armature>();
         QVERIFY(!containsAll);
+
+        QVERIFY(renderer.dirtyBits() & Qt3DRender::Render::AbstractRenderer::AllDirty);
+        renderer.resetDirty();
     }
 
-    void checkRebuildingEntityHierarchy()
+    void checkEntityReparenting()
     {
         // GIVEN
         TestRenderer renderer;
         NodeManagers nodeManagers;
         Qt3DCore::QEntity frontendEntityA, frontendEntityB, frontendEntityC;
 
-        auto entityCreator = [&nodeManagers, &renderer](const Qt3DCore::QEntity &frontEndEntity) {
-            Entity *entity = nodeManagers.renderNodesManager()->getOrCreateResource(frontEndEntity.id());
-            entity->setNodeManagers(&nodeManagers);
-            entity->setRenderer(&renderer);
-            return entity;
-        };
-
-        auto backendA = entityCreator(frontendEntityA);
-        auto backendB = entityCreator(frontendEntityB);
-        auto backendC = entityCreator(frontendEntityC);
+        auto backendA = createEntity(renderer, nodeManagers, frontendEntityA);
+        auto backendB = createEntity(renderer, nodeManagers, frontendEntityB);
+        auto backendC = createEntity(renderer, nodeManagers, frontendEntityC);
 
         // THEN
-        QVERIFY(backendA->parentEntityId().isNull());
-        QVERIFY(backendB->parentEntityId().isNull());
-        QVERIFY(backendC->parentEntityId().isNull());
-
         QVERIFY(backendA->parent() == nullptr);
         QVERIFY(backendB->parent() == nullptr);
         QVERIFY(backendC->parent() == nullptr);
@@ -212,18 +221,13 @@ private slots:
         QVERIFY(backendB->childrenHandles().isEmpty());
         QVERIFY(backendC->childrenHandles().isEmpty());
 
+        QVERIFY(renderer.dirtyBits() & Qt3DRender::Render::AbstractRenderer::AllDirty);
+        renderer.resetDirty();
+
         // WHEN
-        renderer.clearDirtyBits(0);
-        QVERIFY(renderer.dirtyBits() == 0);
-
         auto sendParentChange = [&nodeManagers](const Qt3DCore::QEntity &entity) {
-            const auto parentChange = QPropertyUpdatedChangePtr::create(entity.id());
-            parentChange->setPropertyName("parentEntityUpdated");
-            auto parent = entity.parentEntity();
-            parentChange->setValue(QVariant::fromValue(parent ? parent->id() : Qt3DCore::QNodeId()));
-
             Entity *backendEntity = nodeManagers.renderNodesManager()->getOrCreateResource(entity.id());
-            backendEntity->sceneChangeEvent(parentChange);
+            backendEntity->syncFromFrontEnd(&entity, false);
         };
 
         // reparent B to A and C to B.
@@ -233,12 +237,107 @@ private slots:
         sendParentChange(frontendEntityC);
 
         // THEN
-        QVERIFY(renderer.dirtyBits() & AbstractRenderer::EntityHierarchyDirty);
+        QVERIFY(backendA->parent() == nullptr);
+        QVERIFY(backendB->parent() == backendA);
+        QVERIFY(backendC->parent() == backendB);
 
-        QVERIFY(backendA->parentEntityId().isNull());
-        QVERIFY(backendB->parentEntityId() == frontendEntityA.id());
-        QVERIFY(backendC->parentEntityId() == frontendEntityB.id());
+        QCOMPARE(backendA->childrenHandles().count(), 1);
+        QCOMPARE(backendB->childrenHandles().count(), 1);
+        QVERIFY(backendC->childrenHandles().isEmpty());
 
+        QVERIFY(renderer.dirtyBits() & Qt3DRender::Render::AbstractRenderer::AllDirty);
+        renderer.resetDirty();
+
+          // WHEN - reparent C to A
+        frontendEntityC.setParent(&frontendEntityA);
+        sendParentChange(frontendEntityC);
+
+        // THEN
+        QVERIFY(backendA->parent() == nullptr);
+        QVERIFY(backendB->parent() == backendA);
+        QVERIFY(backendC->parent() == backendA);
+
+        QCOMPARE(backendA->childrenHandles().size(), 2);
+        QVERIFY(backendB->childrenHandles().isEmpty());
+        QVERIFY(backendC->childrenHandles().isEmpty());
+
+        QVERIFY(renderer.dirtyBits() & Qt3DRender::Render::AbstractRenderer::AllDirty);
+        renderer.resetDirty();
+
+        // WHEN - reparent B to null.
+        frontendEntityB.setParent(static_cast<Qt3DCore::QNode *>(nullptr));
+        sendParentChange(frontendEntityB);
+
+        // THEN
+        QVERIFY(backendA->parent() == nullptr);
+        QVERIFY(backendB->parent() == nullptr);
+        QVERIFY(backendC->parent() == backendA);
+
+        QCOMPARE(backendA->childrenHandles().count(), 1);
+        QVERIFY(!backendA->childrenHandles().contains(backendB->handle()));
+        QVERIFY(backendB->childrenHandles().isEmpty());
+        QVERIFY(backendC->childrenHandles().isEmpty());
+
+        QVERIFY(renderer.dirtyBits() & Qt3DRender::Render::AbstractRenderer::AllDirty);
+        renderer.resetDirty();
+      }
+
+    void checkEntityCleanup()
+    {
+        // GIVEN
+        TestRenderer renderer;
+        NodeManagers nodeManagers;
+        Qt3DCore::QEntity frontendEntityA, frontendEntityB, frontendEntityC;
+
+        auto backendA = createEntity(renderer, nodeManagers, frontendEntityA);
+        auto backendB = createEntity(renderer, nodeManagers, frontendEntityB);
+        auto backendC = createEntity(renderer, nodeManagers, frontendEntityC);
+
+        // WHEN
+        auto sendParentChange = [&nodeManagers](const Qt3DCore::QEntity &entity) {
+            Entity *backendEntity = nodeManagers.renderNodesManager()->getOrCreateResource(entity.id());
+            backendEntity->syncFromFrontEnd(&entity, false);
+        };
+
+        // reparent B and C to A.
+        frontendEntityB.setParent(&frontendEntityA);
+        sendParentChange(frontendEntityB);
+        frontendEntityC.setParent(&frontendEntityA);
+        sendParentChange(frontendEntityC);
+
+        // THEN
+        QVERIFY(backendA->parent() == nullptr);
+        QVERIFY(backendB->parent() == backendA);
+        QVERIFY(backendC->parent() == backendA);
+
+        QCOMPARE(backendA->childrenHandles().count(), 2);
+        QVERIFY(backendB->childrenHandles().isEmpty());
+        QVERIFY(backendC->childrenHandles().isEmpty());
+
+        QVERIFY(renderer.dirtyBits() & Qt3DRender::Render::AbstractRenderer::AllDirty);
+        renderer.resetDirty();
+
+        // WHEN - cleaning up a child
+        backendC->cleanup();
+
+        // THEN - the child's parent should be null and it
+        // should be removed from its parent's list of children
+        QVERIFY(backendA->parent() == nullptr);
+        QVERIFY(backendB->parent() == backendA);
+        QVERIFY(backendC->parent() == nullptr);
+
+        QCOMPARE(backendA->childrenHandles().count(), 1);
+        QVERIFY(!backendA->childrenHandles().contains(backendC->handle()));
+        QVERIFY(backendB->childrenHandles().isEmpty());
+        QVERIFY(backendC->childrenHandles().isEmpty());
+
+        QVERIFY(renderer.dirtyBits() & Qt3DRender::Render::AbstractRenderer::AllDirty);
+        renderer.resetDirty();
+
+        // WHEN (cleaning up parent)
+        backendA->cleanup();
+
+        // THEN  (it's children's parent should be set to null)
         QVERIFY(backendA->parent() == nullptr);
         QVERIFY(backendB->parent() == nullptr);
         QVERIFY(backendC->parent() == nullptr);
@@ -247,54 +346,23 @@ private slots:
         QVERIFY(backendB->childrenHandles().isEmpty());
         QVERIFY(backendC->childrenHandles().isEmpty());
 
+        QVERIFY(renderer.dirtyBits() & Qt3DRender::Render::AbstractRenderer::AllDirty);
+        renderer.resetDirty();
+
         // WHEN
-        auto rebuildHierarchy = [](Entity *backend) {
-            backend->clearEntityHierarchy();
-            backend->rebuildEntityHierarchy();
-        };
-        rebuildHierarchy(backendA);
-        rebuildHierarchy(backendB);
-        rebuildHierarchy(backendC);
+        backendB->cleanup();
 
-        // THEN
-        QVERIFY(backendA->parent() == nullptr);
-        QVERIFY(backendB->parent() == backendA);
-        QVERIFY(backendC->parent() == backendB);
-
-        QVERIFY(!backendA->childrenHandles().isEmpty());
-        QVERIFY(!backendB->childrenHandles().isEmpty());
+        // THEN nothing should change
+        QVERIFY(backendA->childrenHandles().isEmpty());
+        QVERIFY(backendB->childrenHandles().isEmpty());
         QVERIFY(backendC->childrenHandles().isEmpty());
 
-        // WHEN - reparent B to null.
-        frontendEntityB.setParent(static_cast<Qt3DCore::QNode *>(nullptr));
-        sendParentChange(frontendEntityB);
-        rebuildHierarchy(backendA);
-        rebuildHierarchy(backendB);
-        rebuildHierarchy(backendC);
-
-        QVERIFY(backendA->parentEntityId().isNull());
-        QVERIFY(backendB->parentEntityId().isNull());
-        QVERIFY(backendC->parentEntityId() == frontendEntityB.id());
-
         QVERIFY(backendA->parent() == nullptr);
         QVERIFY(backendB->parent() == nullptr);
-        QVERIFY(backendC->parent() == backendB);
+        QVERIFY(backendC->parent() == nullptr);
 
-        QVERIFY(backendA->childrenHandles().isEmpty());
-        QVERIFY(!backendB->childrenHandles().isEmpty());
-        QVERIFY(backendC->childrenHandles().isEmpty());
-
-        // WHEN - cleanup
-        backendA->cleanup();
-        backendB->cleanup();
-        backendC->cleanup();
-
-        // THEN
-        QVERIFY(backendA->parentEntityId().isNull());
-        QVERIFY(backendB->parentEntityId().isNull());
-        QVERIFY(backendC->parentEntityId().isNull());
-
-        QVERIFY(renderer.dirtyBits() != 0);
+        QVERIFY(renderer.dirtyBits() & Qt3DRender::Render::AbstractRenderer::AllDirty);
+        renderer.resetDirty();
     }
 
     void shouldHandleSingleComponentEvents_data()
@@ -340,8 +408,7 @@ private slots:
         QVERIFY(method(&entity).isNull());
 
         // WHEN
-        const auto addChange = QComponentAddedChangePtr::create(&dummyFrontendEntity, component);
-        entity.sceneChangeEvent(addChange);
+        EntityPrivate::get(&entity)->componentAdded(component);
 
         // THEN
         QCOMPARE(method(&entity), component->id());
@@ -349,8 +416,7 @@ private slots:
 
         // WHEN
         renderer.resetDirty();
-        const auto removeChange = QComponentRemovedChangePtr::create(&dummyFrontendEntity, component);
-        entity.sceneChangeEvent(removeChange);
+        EntityPrivate::get(&entity)->componentRemoved(component);
 
         // THEN
         QVERIFY(method(&entity).isNull());
@@ -382,7 +448,7 @@ private slots:
     void shouldHandleComponentsEvents()
     {
         // GIVEN
-        QFETCH(QList<QComponent*>, components);
+        QFETCH(const QList<QComponent*>, components);
         QFETCH(void*, functionPtr);
         UuidListMethod method = reinterpret_cast<UuidListMethod>(functionPtr);
 
@@ -395,22 +461,19 @@ private slots:
         QVERIFY(method(&entity).isEmpty());
 
         // WHEN
-        Q_FOREACH (QComponent *component, components) {
-            const auto addChange = QComponentAddedChangePtr::create(&dummyFrontendEntity, component);
-            entity.sceneChangeEvent(addChange);
-        }
+        for (QComponent *component : components)
+            EntityPrivate::get(&entity)->componentAdded(component);
 
         // THEN
         QCOMPARE(method(&entity).size(), components.size());
-        Q_FOREACH (QComponent *component, components) {
+        for (QComponent *component : components) {
             QVERIFY(method(&entity).contains(component->id()));
         }
         QVERIFY(renderer.dirtyBits() != 0);
 
         // WHEN
         renderer.resetDirty();
-        const auto removeChange = QComponentRemovedChangePtr::create(&dummyFrontendEntity, components.first());
-        entity.sceneChangeEvent(removeChange);
+        EntityPrivate::get(&entity)->componentRemoved(components.first());
 
         // THEN
         QCOMPARE(method(&entity).size(), components.size() - 1);
@@ -419,6 +482,126 @@ private slots:
 
         qDeleteAll(components);
     }
+
+    void traversal() {
+        // GIVEN
+        TestRenderer renderer;
+        NodeManagers nodeManagers;
+        Qt3DCore::QEntity frontendEntityA, frontendEntityB, frontendEntityC;
+
+        auto backendA = createEntity(renderer, nodeManagers, frontendEntityA);
+        createEntity(renderer, nodeManagers, frontendEntityB);
+        createEntity(renderer, nodeManagers, frontendEntityC);
+
+        auto sendParentChange = [&nodeManagers](const Qt3DCore::QEntity &entity) {
+            Entity *backendEntity = nodeManagers.renderNodesManager()->getOrCreateResource(entity.id());
+            backendEntity->syncFromFrontEnd(&entity, false);
+        };
+
+        // reparent B to A and C to B.
+        frontendEntityB.setParent(&frontendEntityA);
+        sendParentChange(frontendEntityB);
+        frontendEntityC.setParent(&frontendEntityB);
+        sendParentChange(frontendEntityC);
+
+        // WHEN
+        int visitCount = 0;
+        auto counter = [&visitCount](const Entity *) { ++visitCount; };
+        backendA->traverse(counter);
+
+        // THEN
+        QCOMPARE(visitCount, 3);
+    }
+
+    void visitor()
+    {
+        // GIVEN
+        TestRenderer renderer;
+        NodeManagers nodeManagers;
+        Qt3DCore::QEntity frontendEntityA, frontendEntityB, frontendEntityC;
+
+        frontendEntityA.setEnabled(false);
+        frontendEntityB.setEnabled(false);
+        frontendEntityC.setEnabled(false);
+
+        auto backendA = createEntity(renderer, nodeManagers, frontendEntityA);
+        createEntity(renderer, nodeManagers, frontendEntityB);
+        createEntity(renderer, nodeManagers, frontendEntityC);
+
+        auto sendParentChange = [&nodeManagers](const Qt3DCore::QEntity &entity) {
+            Entity *backendEntity = nodeManagers.renderNodesManager()->getOrCreateResource(entity.id());
+            backendEntity->syncFromFrontEnd(&entity, false);
+        };
+
+        // reparent B to A and C to B.
+        frontendEntityB.setParent(&frontendEntityA);
+        sendParentChange(frontendEntityB);
+        frontendEntityC.setParent(&frontendEntityB);
+        sendParentChange(frontendEntityC);
+
+       // WHEN
+       CompleteVisitor v1(&nodeManagers);
+       EnabledVisitor v2(&nodeManagers);
+       CompleteVisitor v3(&nodeManagers);
+       v3.setPruneDisabled(true);
+       v1.apply(backendA);
+       v2.apply(backendA);
+       v3.apply(backendA);
+
+       // THEN
+       QCOMPARE(v1.count, 3);
+       QCOMPARE(v2.count, 1); // nodes disabled but the first one is still visited before visitation finds out it's disabled
+       QCOMPARE(v3.count, 0); // nodes disabled
+    }
+
+    void accumulator()
+    {
+        // GIVEN
+        TestRenderer renderer;
+        NodeManagers nodeManagers;
+        Qt3DCore::QEntity frontendEntityA, frontendEntityB, frontendEntityC;
+
+        frontendEntityA.setEnabled(false);
+        frontendEntityB.setEnabled(false);
+        frontendEntityC.setEnabled(false);
+
+        auto backendA = createEntity(renderer, nodeManagers, frontendEntityA);
+        createEntity(renderer, nodeManagers, frontendEntityB);
+        createEntity(renderer, nodeManagers, frontendEntityC);
+
+        auto sendParentChange = [&nodeManagers](const Qt3DCore::QEntity &entity) {
+            Entity *backendEntity = nodeManagers.renderNodesManager()->getOrCreateResource(entity.id());
+            backendEntity->syncFromFrontEnd(&entity, false);
+        };
+
+        // reparent B to A and C to B.
+        frontendEntityB.setParent(&frontendEntityA);
+        sendParentChange(frontendEntityB);
+        frontendEntityC.setParent(&frontendEntityB);
+        sendParentChange(frontendEntityC);
+
+        // WHEN
+        EntityAccumulator v1(&nodeManagers);
+        EntityAccumulator v2([](Entity *e) { return e->isEnabled(); }, &nodeManagers);
+        const auto r1 = v1.apply(backendA);
+        const auto r2 = v2.apply(backendA);
+
+        // THEN
+        QCOMPARE(r1.count(), 3);
+        QCOMPARE(r2.count(), 0);
+    }
+
+private:
+    Entity *createEntity(TestRenderer &renderer, NodeManagers &nodeManagers, const Qt3DCore::QEntity &frontEndEntity) {
+        HEntity renderNodeHandle = nodeManagers.renderNodesManager()->getOrAcquireHandle(frontEndEntity.id());
+        Entity *entity = nodeManagers.renderNodesManager()->data(renderNodeHandle);
+        entity->setNodeManagers(&nodeManagers);
+        entity->setHandle(renderNodeHandle);
+        entity->setRenderer(&renderer);
+        entity->syncFromFrontEnd(&frontEndEntity, true);
+        return entity;
+    }
+
 };
 
 QTEST_APPLESS_MAIN(tst_RenderEntity)
